#!/usr/bin/env python3
import os
import requests

from Deadline.Plugins import *
from Deadline.Scripting import *

mistikaProductsDict={ #procuct: [label, binaryName]
    "Boutique":["Mistika Boutique","mistika"],
    "VR":["Mistika VR","vr"],
    "Workflows":["Mistika Workflows","workflows"]
    }
    
MISTIKA_DICT_LABEL=0  
MISTIKA_DICT_BINARY=1
    
mistikaProductKey=os.path.splitext(os.path.basename(__file__).replace("Mistika",""))[0]

def mistikaProductLabel():
    return mistikaProductsDict.get(mistikaProductKey)[MISTIKA_DICT_LABEL]
    
def mistikaProductBinary():
    return mistikaProductsDict.get(mistikaProductKey)[MISTIKA_DICT_BINARY]    

def mistikaProductLongKey():
    global mistikaProductKey
    return "Mistika{}".format(mistikaProductKey)

######################################################################
## This is the function that Deadline calls to get an instance of the
## main DeadlinePlugin class.
######################################################################
def GetDeadlinePlugin():
    return MistikaBoutiquePlugin()

def CleanupDeadlinePlugin( deadlinePlugin ):
    deadlinePlugin.Cleanup()

    
######################################################################
## This is the main DeadlinePlugin class for the Mistika Boutique plugin.
######################################################################
class MistikaBoutiquePlugin( DeadlinePlugin ):
    
    def __init__( self ):
        super().__init__()
        self.InitializeProcessCallback += self.InitializeProcess
        self.RenderExecutableCallback += self.RenderExecutable
        self.RenderArgumentCallback += self.RenderArgument
        self.CheckExitCodeCallback += self.CheckExitCode
        self.renderSize = -1
    
    def Cleanup( self ):
        for stdoutHandler in self.StdoutHandlers:
            del stdoutHandler.HandleCallback

        del self.InitializeProcessCallback
        del self.RenderExecutableCallback
        del self.RenderArgumentCallback
        del self.CheckExitCodeCallback

    ## Called by Deadline to initialize the process.
    def InitializeProcess( self ):
        # Set the plugin specific settings.
        self.PluginType = PluginType.Simple
        self.SingleFramesOnly = False
        self.StdoutHandling = True
        self.PopupHandling = True
        self.HandleQtPopups = True
        self.SetEnvironmentVariable( "QT_USE_NATIVE_WINDOWS", "1" )

        self.AddPopupIgnorer( "BackGround render of.*" )
        # this closes the popup for a license error
        self.AddPopupHandler(mistikaProductLabel(), "[X]" ) 

        self.AddStdoutHandlerCallback( "render error:(.*)" ).HandleCallback += self.HandleRenderError
        self.AddStdoutHandlerCallback( "Check your license server for available licenses." ).HandleCallback += self.HandleLicenseError
        self.AddStdoutHandlerCallback( "^SIZE ([0-9]+)$" ).HandleCallback += self.HandleRenderSize
        self.AddStdoutHandlerCallback( "Waited [0-9]+ ms to write frame ([0-9]+).*" ).HandleCallback += self.HandleProgress
        self.AddStdoutHandlerCallback( "Done: ([0-9]+).*" ).HandleCallback += self.HandleProgress
        #self.AddStdoutHandlerCallback( "Starting " ).HandleCallback += self.HandleStarting
        #self.AddStdoutHandlerCallback( "total time: " ).HandleCallback += self.HandleFinished

    def RenderExecutable( self ):
        executableList = self.GetConfigEntry( "{}Executable".format(mistikaProductKey))
        executable = FileUtils.SearchFileList( executableList )

        if not executable:
            self.FailRender( mistikaProductLabel()+" render executable was not found in the semicolon separated list \"" + executableList + "\". The path to the Mistika render executable can be configured from the Plugin Configuration in the Deadline Monitor." )
        
        return executable

    def HandlePathSeparators( self, filePath ):
        if os.name == "nt":
            filePath = filePath.replace( "/", "\\" )
            if filePath.startswith( "\\" ) and not filePath.startswith( "\\\\" ):
                filePath = "\\" + filePath
        else:
            filePath = filePath.replace( "\\", "/" )

        return filePath

    def RenderArgument( self ):
        sceneFile = self.GetPluginInfoEntryWithDefault( "SceneFile", self.GetDataFilename() )
        sceneFile = RepositoryUtils.CheckPathMapping( sceneFile )
        sceneFile = self.HandlePathSeparators( sceneFile )
        blockSize = int(self.GetPluginInfoEntryWithDefault( "blockSize","0"))
            
        
        # Path map the contents of the Mistika render file
        if False and self.GetBooleanConfigEntryWithDefault( "EnablePathMapping", False ):
            tempSceneDirectory = self.CreateTempDirectory( "thread" + str( self.GetThreadNumber() ) )
            tempSceneFile = os.path.join( tempSceneDirectory, os.path.basename( sceneFile ) )

            if os.name == "nt":
                RepositoryUtils.CheckPathMappingInFile( sceneFile, tempSceneFile )
            else:
                tempSceneFile = tempSceneFile.replace( "\\", "/" )
                RepositoryUtils.CheckPathMappingInFile( sceneFile, tempSceneFile )
                os.chmod( tempSceneFile, os.stat( sceneFile ).st_mode )

            sceneFile = tempSceneFile

        renderArguments = "--verbose -q -r \"" + sceneFile + "\" "

        startFrame = str( self.GetStartFrame() )
        endFrame = str( self.GetEndFrame() )
    
        if blockSize>0:
            renderArguments += "-s " + startFrame + " -e " + endFrame

        restURL = self.GetPluginInfoEntryWithDefault( "restServer", "")
        jobID = self.GetPluginInfoEntryWithDefault( "jobID", "")
        if restURL and jobID:
            renderArguments += ' --rest-server=\"{}\"'.format(restURL)
            renderArguments += ' --job-id=\"{}\"'.format(jobID)
        return renderArguments

    def CheckExitCode( self, exitCode ):
        # SGO previously used exit code 2 as a different way to signal a success, and will not be used for any error code in the future
        if exitCode not in ( 0, 2 ):
            self.sendFeedback("finished?status={}".format(exitCode))
            self.FailRender( "Process exit code: {}".format(exitCode))

    def HandleRenderError( self ):
        errorMessage = self.GetRegexMatch( 1 )
        self.FailRender( "Detected a render error: %s" % ( errorMessage.strip() ) )

    def HandleLicenseError( self ):
        self.FailRender( "Invalid {} license. {}".format(mistikaProductLabel,self.GetRegexMatch(0)))

    def HandleRenderSize( self ):
        self.renderSize = float( self.GetRegexMatch( 1 ) )

    def HandleProgress( self ):
        progress = 0
        # strip leading 0s so Python doesn't think the frame number is an octal
        str=self.GetRegexMatch( 1 ).lstrip( "0" ) 
        currentFrame = 0 if str=="" else int(str)
        imageRender = self.GetBooleanPluginInfoEntry( "ImageRender" )
        url = self.GetPluginInfoEntryWithDefault("restServer","")
        jobID = self.GetPluginInfoEntryWithDefault("jobID","")
        firstFrame=int(self.GetStartFrame())
        if imageRender:
            firstFrame=int(self.GetStartFrame())
            framesCompletedInTask = currentFrame - self.GetStartFrame() + 1
            totalFrames = int( self.GetEndFrame() ) - firstFrame + 1
            progress = float( framesCompletedInTask ) / float( totalFrames ) * 100
        else:
            if self.renderSize == -1:
                # callback to set renderSize hasn't kicked in yet
                return
            progress = float( currentFrame ) / self.renderSize * 100
            
#        if url and jobID:
#            api_url = "{}/renderFarm/job/{}/progress?currentFrame={}&firstFrame={}".format(url,jobID,int(currentFrame),firstFrame)
#            requests.post(api_url)
        self.SetProgress( progress )

    def HandleStarting( self ):

        url = self.GetPluginInfoEntryWithDefault("restServer","")
        jobID = self.GetPluginInfoEntryWithDefault("jobID","")
        lastFrame=int(self.GetEndFrame())
        self.sendFeedback("progress?lastFrame={}".format(lastFrame))

    def HandleFinished( self ):
        url = self.GetPluginInfoEntryWithDefault("restServer","")
        jobID = self.GetPluginInfoEntryWithDefault("jobID","")
        status=0
        self.sendFeedback("finished?status={}".format(status))

    def sendFeedback( self,cmd):
        url = self.GetPluginInfoEntryWithDefault("restServer","")
        jobID = self.GetPluginInfoEntryWithDefault("jobID","")
        firstFrame=int(self.GetStartFrame())
        if url and jobID:
            api_url = "{}/renderFarm/job/{}/{}&firstFrame={}".format(url,jobID,cmd,firstFrame)
            print("Feedback:",url,jobID,cmd,firstFrame)
            print(api_url)
            try:
                requests.post(api_url)
            except:
                pass